/*  Multiple precision integer arithmetic - see multip1.notes	*/
/*  Source:  miarith.c		Header:  miarith.h		*/
/*  Ernest Sibert -- February 1995				*/

/* -------------------------------------------------------------------- */

/*  Divide 2-word non-negative integer [d1, d0] by 1-word, normalized	*/
/*  non-negative integer x, returning 1-word quotient q and storing 	*/
/*  1-word remainder r.  This assumes d1 < x .				*/


extern int idiv2by1(int d1, int d0, int x, int *r);

/* -------------------------------------------------------------------- */

/*  x[0..N] := x[0..N] div s, sets r to remainder, 0 <= r < s	*/
/*  Requires x >= 0,  0 < s < 2^Mhalf [ 2^15 ]			*/

extern void imdivs(int x[], int N, int s,  int *r);

/* -------------------------------------------------------------------- */

/*  Non-negative integer division for normalized divisors, 	*/
/*  M >= 2.  Computes q and r according to			*/
/*								*/
/*	   dd[0..M+N] = x[0..M]*q[0..N] + r[0..M]		*/
/*  where							*/
/*	   M >= 2,  N >= 0,					*/
/*	   dd[0..M+N] >= 0,					*/
/*	   x[1] >= 2^(Mword - 1)  (so x > 0),			*/
/*	   x[0..M] > dd[0..M]  (so quotient fits),		*/
/*	   and  0 <= r < |x|,					*/
/*	   p[0..M+1] is working array for partial products	*/
/*  								*/
/*  Leaves r pointing to remainder which resides in dd[N..M+N]	*/
/*								*/
/*  It follows that the quotient will fit in N words.		*/
/*  dd will be altered by the calculation.			*/
/*  The division cannot fail if x and dd satisfy the spec.	*/
/*  None of dd[0..M+N], x[0..M], q[0..N], r[0..M], p[0..M+1]	*/
/*  should interfere.						*/

extern
 void imdiv2x(int dd[], int x[], int q[],  int **r,  int p[], int M, int N);

/* -------------------------------------------------------------------- */

/*  General integer division, excluding most-negative integer.	*/
/*  Computes q and r according to				*/
/*								*/
/*	   dd[0..M+N] = x[0..M]*q[0..N+1] + r[0..M]		*/
/*  where							*/
/*	   |x| != 0,  |r| < |x|  and  r*dd >= 0			*/
/*								*/
/*  It is required that the quotient fit in N+1 words.		*/
/*  Both q and r show overflows if the division fails.		*/
/*  None of x[0..M], q[0..N+1], r[0..M] should interfere,	*/
/*  but they may interfere with dd[0..M+N].			*/

extern void imdiv2(int dd[], int x[], int q[],  int r[],  int M, int N);

/* -------------------------------------------------------------------- */
